local addon = LibStub("AceAddon-3.0"):GetAddon("RCLootCouncil")
local RCEPGP = addon:NewModule("RCEPGP", "AceComm-3.0", "AceConsole-3.0", "AceHook-3.0", "AceEvent-3.0")
local EPGP = LibStub("AceAddon-3.0"):GetAddon("EPGP")
local GS = LibStub("LibGuildStorage-1.2")
local L = LibStub("AceLocale-3.0"):GetLocale("RCLootCouncil")
local GP = LibStub("LibGearPoints-1.2")
local LibDialog = LibStub("LibDialog-1.0")
local RCLootCouncilML = addon:GetModule("RCLootCouncilML")

local votingFrame = addon:GetActiveModule("votingframe")
local originalCols = {unpack(votingFrame.scrollCols)}

local session = 1

function RCEPGP:OnInitialize()
   self.version = GetAddOnMetadata("RCLootCouncil_EPGP", "Version")
   self:Enable()
end

function RCEPGP.UpdateVotingFrame()
  votingFrame:Update()
end

function RCEPGP:OnEnable()
   EPGP.RegisterCallback(self, "StandingsChanged",
                        RCEPGP.UpdateVotingFrame)
   self:SetupColumns()
   self:AddGPOptions()
   addon:SecureHook(RCVotingFrame, "OnEnable", self.AddGPEditBox)
   addon:SecureHook(RCVotingFrame, "SwitchSession", self.AddGPSwitchSession)
   Lib_UIDropDownMenu_Initialize(RCLootCouncil_VotingFrame_RightclickMenu, self.RightClickMenu, "MENU")
end

function RCEPGP:OnDisable()
   -- Reset cols
   votingFrame.scrollCols = originalCols
end


function RCEPGP:SetupColumns()
   local ep =
   { name = "EP", DoCellUpdate = self.SetCellEP, colName = "ep", sortnext = self.GetScrollColIndexFromName("response"), width = 60, align = "CENTER", }
   local gp =
   { name = "GP", DoCellUpdate = self.SetCellGP, colName = "gp", sortnext = self.GetScrollColIndexFromName("response"), width = 50, align = "CENTER", }
   local pr =
   { name = "PR", DoCellUpdate = self.SetCellPR, colName = "pr", sortnext = self.GetScrollColIndexFromName("response"), width = 50, align = "CENTER", comparesort = self.PRSort,}
   table.insert(votingFrame.scrollCols, ep)
   table.insert(votingFrame.scrollCols, gp)
   table.insert(votingFrame.scrollCols, pr)
end

function RCEPGP:GetScrollColIndexFromName(name)
   for i,v in ipairs(votingFrame.scrollCols) do
      if v.colName == name then
         return i
      end
   end
end

---------------------------------------------
-- Lib-st UI functions
---------------------------------------------
function RCEPGP.SetCellEP(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
    local name = data[realrow].name
    local ep, gp, main = EPGP:GetEPGP(name)
    frame.text:SetText(ep or 0)
    data[realrow].cols[column].value = ep or 0
end

function RCEPGP.SetCellGP(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
    local name = data[realrow].name
    local ep, gp, main = EPGP:GetEPGP(name)
    frame.text:SetText(gp or 0)
    data[realrow].cols[column].value = gp or 0
end

function RCEPGP.SetCellPR(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
    local name = data[realrow].name
    local ep, gp, main = EPGP:GetEPGP(name)
    local pr = 0
    if ep and gp then
        pr = ep / gp
    end
    frame.text:SetText(string.format("%.4f", pr))
    data[realrow].cols[column].value = pr or 0
end

function RCEPGP.PRSort(table, rowa, rowb, sortbycol)
  local column = table.cols[sortbycol]
  local a, b = table:GetRow(rowa), table:GetRow(rowb);
  -- Extract the rank index from the name, fallback to 100 if not found

  local a_ep, a_gp = EPGP:GetEPGP(a.name)
  local b_ep, b_gp = EPGP:GetEPGP(b.name)

  a_ep = a_ep or 0
  a_gp = a_gp or 1
  b_ep = b_ep or 0
  b_gp = b_gp or 1

  local a_pr = a_ep/a_gp
  local b_pr = b_ep/b_gp

  local a_qualifies = a_ep >= EPGP.db.profile.min_ep
  local b_qualifies = b_ep >= EPGP.db.profile.min_ep

  if a_qualifies == b_qualifies and a_pr == b_pr then
    if column.sortnext then
      local nextcol = table.cols[column.sortnext];
      if nextcol and not(nextcol.sort) then
        if nextcol.comparesort then
          return nextcol.comparesort(table, rowa, rowb, column.sortnext);
        else
          return table:CompareSort(rowa, rowb, column.sortnext);
        end
      end
    end
    return false
  else
    local direction = column.sort or column.defaultsort or "asc";
    if direction:lower() == "asc" then
      if a_qualifies == b_qualifies then
        return a_pr < b_pr
      else
        return b_qualifies
      end
    else
      if a_qualifies == b_qualifies then
        return a_pr > b_pr
      else
        return a_qualifies
      end
    end
  end
end

----------------------------------------------------------------
function RCEPGP:AddGPOptions()
  local options = addon:OptionsTable()

  addon.options = addon:OptionsTable()
  local button, picker, text, gp = {}, {}, {}, {}
  for i = 1, addon.db.profile.maxButtons do
      addon.options.args.mlSettings.args.buttonsTab.args.buttonOptions.args["button" .. i].order = i * 4 + 1;
      addon.options.args.mlSettings.args.buttonsTab.args.buttonOptions.args["picker" .. i].order = i * 4 + 2;
      addon.options.args.mlSettings.args.buttonsTab.args.buttonOptions.args["picker" .. i].width = "half";
      addon.options.args.mlSettings.args.buttonsTab.args.buttonOptions.args["text" .. i].order = i * 4 + 3;
      gp = {
          order = i * 4 + 4,
          name = "GP",
          desc = "Example:\r\n100%: use 100% of normal GP value\r\n50%: use 50% of normal GP value \r\n25: all items are worth 25 GP",
          type = "input",
          width = "half",
          get = function() return addon.db.profile.responses[i].gp or "100%" end,
          set = function(info, value) 
            if not value then value = "100%" end
            value = tostring(value)
            if string.match(value, "^%d+%%$") or string.match(value, "^%d+$") then
              addon.db.profile.responses[i].gp = tostring(value) 
            end
          end,
          hidden = function() return addon.db.profile.numButtons < i end,
      }
      if not addon.db.profile.responses[i].gp then
        addon.db.profile.responses[i].gp = "100%"
      end
      addon.options.args.mlSettings.args.buttonsTab.args.buttonOptions.args["gp" .. i] = gp;
  end


  for k, v in pairs(addon.db.profile.responses.tier) do
      addon.options.args.mlSettings.args.buttonsTab.args.tierButtonsOptions.args["button" .. k].order = v.sort * 4 + 1
      addon.options.args.mlSettings.args.buttonsTab.args.tierButtonsOptions.args["color" .. k].order = v.sort * 4 + 2
      addon.options.args.mlSettings.args.buttonsTab.args.tierButtonsOptions.args["color" .. k].width = "half"
      addon.options.args.mlSettings.args.buttonsTab.args.tierButtonsOptions.args["text" .. k].order = v.sort * 4 + 3
      gp = {
          order = v.sort * 4 + 4,
          name = "GP",
          desc = "Example:\r\n100%: use 100% of normal GP value\r\n50%: use 50% of normal GP value \r\n25: all items are worth 25 GP",
          type = "input",
          width = "half",
          get = function() return addon.db.profile.tierButtons[v.sort].gp or "100%" end,
          set = function(info, value) 
            if not value then value = "100%" end
            value = tostring(value)
            if string.match(value, "^%d+%%$") or string.match(value, "^%d+$") then
              addon:ConfigTableChanged("responses");addon.db.profile.tierButtons[v.sort].gp = tostring(value) 
            end
            end,
          hidden = function() return not addon.db.profile.tierButtonsEnabled or addon.db.profile.tierNumButtons < v.sort end,
      }
      if not addon.db.profile.tierButtons[v.sort].gp then
          addon.db.profile.tierButtons[v.sort].gp = "100%"
      end
      addon.options.args.mlSettings.args.buttonsTab.args.tierButtonsOptions.args["gp" .. k] = gp
  end


  addon.options.args.settings.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.db)
  LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("RCLootCouncil", addon.options)
  LibStub("AceConfigRegistry-3.0"):NotifyChange("RCLootCouncil")
  addon:ConfigTableChanged("buttons");
  addon:ConfigTableChanged("tierButtons"); 
end

function RCEPGP:AddGPEditBox()
  if not RCVotingFrame.frame.gpString then
      local gpstr = RCVotingFrame.frame.content:CreateFontString(nil, "OVERLAY", "GameFontNormal")
      gpstr:SetPoint("CENTER", RCVotingFrame.frame.content, "TOPLEFT", 300, -60)
      gpstr:SetText("GP: ")
      gpstr:Show()
      gpstr:SetTextColor(1, 1, 0, 1) -- Yellow
      RCVotingFrame.frame.gpString = gpstr
  end


  local editbox_name = "RCLootCouncil_GP_EditBox"
  if not RCVotingFrame.frame.editbox then
      local editbox = _G.CreateFrame("EditBox", editbox_name, RCVotingFrame.frame.content, "AutoCompleteEditBoxTemplate")
      editbox:SetWidth(32)
      editbox:SetHeight(32)
      editbox:SetFontObject("ChatFontNormal")
      editbox:SetNumeric(true)
      editbox:SetMaxLetters(4)

      local left = editbox:CreateTexture(("%sLeft"):format(editbox_name), "BACKGROUND")
      left:SetTexture([[Interface\ChatFrame\UI-ChatInputBorder-Left2]])
      left:SetWidth(8)
      left:SetHeight(32)
      left:SetPoint("LEFT", -5, 0)

      local right = editbox:CreateTexture(("%sRight"):format(editbox_name), "BACKGROUND")
      right:SetTexture([[Interface\ChatFrame\UI-ChatInputBorder-Right2]])
      right:SetWidth(8)
      right:SetHeight(32)
      right:SetPoint("RIGHT", 5, 0)

      local mid = editbox:CreateTexture(("%sMid"):format(editbox_name), "BACKGROUND")
      mid:SetTexture([[Interface\ChatFrame\UI-ChatInputBorder-Mid2]])
      mid:SetHeight(32)
      mid:SetPoint("TOPLEFT", left, "TOPRIGHT", 0, 0)
      mid:SetPoint("TOPRIGHT", right, "TOPLEFT", 0, 0)

      --local label = editbox:CreateFontString(editbox_name, "ARTWORK", "GameFontNormalSmall")
      --label:SetPoint("RIGHT", editbox, "LEFT", - 15, 0)
      --label:Show()
      editbox.left = left
      editbox.right = right
      editbox.mid = mid
      --editbox.label = label

      editbox:SetPoint("LEFT", RCVotingFrame.frame.gpString, "RIGHT", 10, 0)
      editbox:Show()
      editbox:SetScript("OnTextChanged", function(self, userInput) RCVotingFrame:Update() end)
      RCVotingFrame.frame.editbox = editbox
  end
end

function RCEPGP:AddGPSwitchSession(s)
  local lootTable = RCVotingFrame:GetLootTable()
  session = s
  local t = lootTable[s]
  local gp = GP:GetValue(t.link) or 0
  self.frame.editbox:SetNumber(gp)
end

function RCEPGP:GetResponseGP(response, isTier)
  if response == "PASS" or response == "AUTOPASS" then
    return "0%"
  end
  local responseGP = "100%"

  local index = addon:GetResponseSort(response, isTier)
  if isTier and addon.mldb.tierButtonsEnabled then
    for k, v in pairs(addon.db.profile.tierButtons) do 
      if v.text == response then
        responseGP = v.gp or responseGP
        break
      elseif k == response then
        responseGP = v.gp or responseGP
        break
      end
    end
  else
    for k, v in pairs(addon.db.profile.responses) do 
      if v.text == response then
        responseGP = v.gp or responseGP
        break
      elseif k == response then
        responseGP = v.gp or responseGP
        break
      end
    end
  end
  return responseGP
end

function RCEPGP.RightClickMenu(menu, level)
  if not addon.isMasterLooter then return end
  local lootTable = RCVotingFrame:GetLootTable()
  local candidateName = menu.name
  local data = lootTable[session].candidates[candidateName]
  local responseGP = RCEPGP:GetResponseGP(data.response, data.isTier)

  local editboxGP = RCVotingFrame.frame.editbox:GetNumber()
  local gp
  if string.match(responseGP, "^%d+$") then
    gp = tonumber(responseGP)
  else -- responseGP is percentage like 55%
    local coeff = tonumber(string.match(responseGP, "%d+"))/100
    gp = math.floor(coeff * editboxGP)
  end

  if level == 1 then
    info = Lib_UIDropDownMenu_CreateInfo()
    info.text = L["Award"].." ("..gp.." GP)"
    if string.match(responseGP, "^%d+%%") then
      info.text = L["Award"].." ("..gp.." GP, "..responseGP..")"
    end
    info.func = function()
      LibDialog:Spawn("RCEPGP_CONFIRM_AWARD", {
        session,
        candidateName,
        data.response,
        nil,
        data.votes,
        data.gear1,
        data.gear2,
        data.isTier,
        gp,
        responseGP
    }) end
    info.disabled = false
    local item = RCLootCouncilML.lootTable[session].link
    if not EPGP:CanIncGPBy(item, gp) then
      info.disabled = true
    end
    Lib_UIDropDownMenu_AddButton(info, level)
  end
  RCVotingFrame.RightClickMenu(menu, level)
end


LibDialog:Register("RCEPGP_CONFIRM_AWARD", {
  text = "something_went_wrong",
  icon = "",
  on_show = function(self, data)
    local session, player, response, reason, votes, item1, item2, isTierRoll, gp, responseGP = unpack(data,1,10)
    self:SetFrameStrata("FULLSCREEN")
    local session, player = unpack(data)
    self.text:SetText(format(L["Are you sure you want to give #item to #player?"].." ("..gp.." GP)", RCLootCouncilML.lootTable[session].link, addon.Ambiguate(player)))
    if string.match(responseGP, "^%d+%%") then
        self.text:SetText(format(L["Are you sure you want to give #item to #player?"].." ("..gp.." GP, "..responseGP.."%"..")", RCLootCouncilML.lootTable[session].link, addon.Ambiguate(player)))
    end
    self.icon:SetTexture(RCLootCouncilML.lootTable[session].texture)
  end,
  buttons = {
    { text = L["Yes"],
      on_click = function(self, data)
        -- IDEA Perhaps come up with a better way of handling this
        local session, player, response, reason, votes, item1, item2, isTierRoll, gp,responseGP = unpack(data,1,9)
        local item = RCLootCouncilML.lootTable[session].link -- Store it now as we wipe lootTable after Award()
        local isToken = RCLootCouncilML.lootTable[session].token
        local awarded = RCLootCouncilML:Award(session, player, response, reason, isTierRoll)
        if awarded then -- log it
          RCLootCouncilML:TrackAndLogLoot(player, item, response, addon.target, votes, item1, item2, reason, isToken, isTierRoll)
          EPGP:IncGPBy(player, item, gp)
        end
        -- We need to delay the test mode disabling so comms have a chance to be send first!
        if addon.testMode and RCLootCouncilML:HasAllItemsBeenAwarded() then RCLootCouncilML:EndSession() end
      end,
    },
    { text = L["No"],
    },
  },
  hide_on_escape = true,
  show_while_dead = true,
})

